//******************************************************************************
//******************************************************************************
/**
 * @file    IBPNGClientListener.h
 * @author  Markus van Pinxteren
 * @date    12.05.2010
 *
 * @brief   Interface class for the BPNGClient listener
*/
// $Id$
//******************************************************************************
//******************************************************************************

#ifndef __IBPNGCLIENTLISTENER_HH__
#define __IBPNGCLIENTLISTENER_HH__

#ifdef _WINDOWS
#include <windows.h>
#endif
#include "BPNGDefines.h"

extern "C" {
struct IBPNGClientListener
{
	/// Called to notify a detected logger in network
	/**
	  All char* of the passed OnlineLoggerInfo* are only valid for the 
	  time of the function call. Please ensure to copy the string values.
	*/
	virtual void WINAPI onBPNGDeviceDetected(OnlineLoggerInfo* info) = 0;

	/// Called to notify a disappeared logger
	/**
	  All char* of the passed OnlineLoggerInfo* are only valid for the 
	  time of the function call. Please ensure to copy the string values.
	*/
	virtual void WINAPI onBPNGDeviceDisappeared(OnlineLoggerInfo* info) = 0;

	/// Called to notify a logger's state change
	/**
	  All char* of the passed OnlineLoggerInfo* are only valid for the 
	  time of the function call. Please ensure to copy the string values.
	*/
	virtual void WINAPI onBPNGDeviceStateChange(OnlineLoggerInfo* info) = 0;

	/// Called to indicate the current progress of a file transfer
	/**
	  This function notifies the listener about the download progress of the raw Telemotive trace data.

	  @param percentCompleted Percentage of the entire download process (from 0...100%). 
	                          A negative value can be passed if we only want to check a abort request
	  @return return value 0 indicates an abort request from the implementing class
	*/
	virtual int WINAPI onProgressDataDownload(int percentCompleted) = 0;

	/// Called to indicate the current progress of file conversion
	/**
	  This function notifies the listener about the conversion progress of the raw Telemotive trace data.
	  If the \a percentCompleted value has changed, but the \a status is still the
	  same, the application passes an empty string as status to the function.

	  @param percentCompleted Percent of the entire conversion process (from 0...100%), -1 indicates the same value as from last function call
	  @param status Status of the conversion process (e.g. "Converting trace data. Block 5 of 32")
	  @return return value 0 indicates an abort request from the implementing class
	*/
	virtual int WINAPI onProgressConversion(int percentCompleted, const char* status) = 0;

	/// Called to send additional information of the current process to the calling app
	/**
	  This function transmit message strings to the listener class. Those messages are only
	  for imformation purpose. The receiver doesn't have to react on it but can display it
	  on the screen.
	*/
	virtual void WINAPI onStatusMessage(const char* statusMsg) = 0;
	
	/// Called to inform about a warning.
	/**
	  This function transmit a warning message to the listener class. Warnings have a WARING_CODE and a warning message.
	  Warnings do not interrupt the current process but should be noticed from the user to possibly initiate further provisions.
	*/
	virtual void WINAPI onWarning(BPNGWarningCode warningCode, const char* warnMsg) = 0;

	/// Called on a too long target directory
	/** 
	  Called when the resulting file name of the converted files is longer than the maximum allowed size
	  of the file system (Windows 260). The lib user has to pass a new (shorter) base target directory to the passed char
	  array with strcpy. The memory of the array is already allocated by the library and it's size is maxSize.
	  When a new directory was set the value 1 must be returned. If a listener wants to ignore this function call 
	  0 has to be returned.
	*/
	virtual int WINAPI onTargetPathTooLong(char* newTarget, int maxSize) = 0;

	/// Called on existing output trace files
	/**
	  When an output trace file already exists this function is called. The listener has the possibility to
	  return one of following values:
	  -1: no, don't overwrite file
	  -2: no, overwrite neither this nor any following file
	   1: yes, overwrite file
	   2: yes, overwrite this and all following files
	   0: cancel conversion
	*/
	virtual int WINAPI getOverwritingPermission(const char* filePath) = 0;

	/// Called in case of not enough free diskspace
	/**
		This notifies the listener about not enough free disk space for data download or conversion.
		The user can continue or abort the process. Returning 0 will abort the process.
		In some cases continuing without providing more disk space will call this function
		immediately again.
		@param freeSpace Amount of free space
		@param neededSpace Amount of needed space
		@param drive Name of the drive where to store data
		@param msg Additional message to display
		@return return 0 when process should be aborted, 1 to ignore
	*/
	virtual int WINAPI onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char* drive, const char* msg) = 0;

	/// Called on firmware update progress
	virtual void WINAPI onFirmwareUpdateProgress(int percentage, int stepId, int subStepId, const char* desc) = 0;
	virtual void WINAPI onFirmwareUpdateError(int  errorId) = 0;

	/// Called on creation of log report
	/// @return return value 0 indicates an abort request from the implementing class
	virtual int WINAPI onGetLogReportProgress(int percentage, const char* desc) = 0;
};

} // extern C

#endif // __IBPNGCLIENTLISTENER_HH__
